/* GnoBang: (c) 2008 by Andreas S. Krebs
 * 
 * This file is part of GnoBang.
 *
 * GnoBang is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GnoBang is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GnoBang.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "gnobang.h"

/* the default settings */
static GoGame	DefaultGoGame=
{
	{				// Player[2]
		{
			PLAYER_HUMAN,		// Type
			PLAY_NORMAL,		// PlayMode
			0xc80000,			// Color
			"You",				// Name
			0					// Seconds
		},
		{
			PLAYER_COMPUTER,	// Type
			PLAY_NORMAL,		// PlayMode
			0x0000c8,			// Color
			"Me",				// Name
			0					// Seconds
		}
	},
	0,				// PlayersTurn
	0,				// LastWinner
	START_PLAYER1,	// StartPlayer
	0x007800,		// BoardColor
	0x000000,		// BoardBackground
	0x787800,		// HintColor
	GAME_NEW,		// GameStatus
	20,15,			// Columns, Rows
 	400,400,		// Width, Height
	CHEAT_NONE,		// CheatMode
	TRUE			// SoundOn
};


/************************************************************************
 *																		*
 * init pref dialog with given prefs								 	*
 *  																	*
 ************************************************************************/

static void initPrefsDialog(GoGame *gg)
{
	GdkColor	boardCol, bgCol, hintCol, playerCol[2];

	gtk_spin_button_set_value(gnoBang->columnsButton, gg->Columns);
	gtk_spin_button_set_value(gnoBang->rowsButton, gg->Rows);
	gtk_combo_box_set_active(gnoBang->firstMoveBox, gg->StartPlayer);
	gtk_combo_box_set_active(gnoBang->cheatModeBox, gg->CheatMode);

#ifdef SOUND_SUPPORT
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gnoBang->soundCheckButton), gg->SoundOn);
#else
	gtk_widget_set_child_visible(GTK_WIDGET(gnoBang->soundCheckButton), FALSE);
#endif
	
	gtk_combo_box_set_active(gnoBang->playerTypeBox[0], gg->Player[0].Type);
	gtk_entry_set_text(gnoBang->playerNameField[0], gg->Player[0].Name);
	gtk_combo_box_set_active(gnoBang->playerStrategyBox[0], gg->Player[0].PlayMode);
  
	gtk_combo_box_set_active(gnoBang->playerTypeBox[1], gg->Player[1].Type);
	gtk_entry_set_text(gnoBang->playerNameField[1], gg->Player[1].Name);
	gtk_combo_box_set_active(gnoBang->playerStrategyBox[1], gg->Player[1].PlayMode);
  
	/* set colors */
	setGdkColor(gnoBang->drawingArea, &boardCol, gg->BoardColor);
	gtk_color_button_set_color(gnoBang->boardColorButton, &boardCol);
	setGdkColor(gnoBang->drawingArea, &bgCol, gg->BoardBackground);
	gtk_color_button_set_color(gnoBang->backgroundColorButton, &bgCol);
	setGdkColor(gnoBang->drawingArea, &hintCol, gg->HintColor);
	gtk_color_button_set_color(gnoBang->hintColorButton, &hintCol);
	setGdkColor(gnoBang->drawingArea, &playerCol[0], gg->Player[0].Color);
	gtk_color_button_set_color(gnoBang->playerColorButton[0], &playerCol[0]);
	setGdkColor(gnoBang->drawingArea, &playerCol[1], gg->Player[1].Color);
	gtk_color_button_set_color(gnoBang->playerColorButton[1], &playerCol[1]);	
}


/************************************************************************
 *																		*
 * callback for preferences menu item								 	*
 *  																	*
 ************************************************************************/

void preferencesCB(GtkObject *object)
{
	int rc;

	/* init dialog before showing it */
	initPrefsDialog(goGame);
	
	do
	{
		rc=gtk_dialog_run(gnoBang->prefDiag);
	}while(rc==0);	// loop until we get a code from a button we're interested in
	
	if(rc==GTK_RESPONSE_OK)
	{
		GdkColor	boardCol, bgCol, hintCol, playerCol[2];
		const char	*s;
		
		/* dialog close with okay, so get new values */
		goGame->Columns=gtk_spin_button_get_value_as_int(gnoBang->columnsButton);
		goGame->Rows=gtk_spin_button_get_value_as_int(gnoBang->rowsButton);
		goGame->StartPlayer=gtk_combo_box_get_active(gnoBang->firstMoveBox);
		goGame->CheatMode=gtk_combo_box_get_active(gnoBang->cheatModeBox);
		goGame->SoundOn=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gnoBang->soundCheckButton));
		
		goGame->Player[0].Type=gtk_combo_box_get_active(gnoBang->playerTypeBox[0]);
		goGame->Player[0].PlayMode=gtk_combo_box_get_active(gnoBang->playerStrategyBox[0]);
		s=(const char *)gtk_entry_get_text(gnoBang->playerNameField[0]);
		strncpy(goGame->Player[0].Name, s, sizeof(goGame->Player[0].Name));
		goGame->Player[0].Name[sizeof(goGame->Player[0].Name)-1]=0;
		
		goGame->Player[1].Type=gtk_combo_box_get_active(gnoBang->playerTypeBox[1]);
		goGame->Player[1].PlayMode=gtk_combo_box_get_active(gnoBang->playerStrategyBox[1]);
		s=(const char *)gtk_entry_get_text(gnoBang->playerNameField[1]);
		strncpy(goGame->Player[1].Name, s, sizeof(goGame->Player[1].Name));
		goGame->Player[1].Name[sizeof(goGame->Player[1].Name)-1]=0;
		
		/* now get the colors */
		gtk_color_button_get_color(gnoBang->boardColorButton, &boardCol);
		goGame->BoardColor=getGdkColor(&boardCol);
		gtk_color_button_get_color(gnoBang->backgroundColorButton, &bgCol);
		goGame->BoardBackground=getGdkColor(&bgCol);
		gtk_color_button_get_color(gnoBang->hintColorButton, &hintCol);
		goGame->HintColor=getGdkColor(&hintCol);
		gtk_color_button_get_color(gnoBang->playerColorButton[0], &playerCol[0]);
		goGame->Player[0].Color=getGdkColor(&playerCol[0]);
		gtk_color_button_get_color(gnoBang->playerColorButton[1], &playerCol[1]);
		goGame->Player[1].Color=getGdkColor(&playerCol[1]);
		
		/* finally trigger the update to the board */
		applyPreferences();
	}

	gtk_widget_hide(GTK_WIDGET(gnoBang->prefDiag));
}


/************************************************************************
 *																		*
 * callback for setting default preferences							 	*
 *  																	*
 ************************************************************************/

void defaultPrefsCB(GtkObject *object)
{
	initPrefsDialog(&DefaultGoGame);
}


/************************************************************************
 *																		*
 * callback for changing player type								 	*
 *  																	*
 ************************************************************************/

void typeChangedCB(GtkObject *object)
{
	int			type=gtk_combo_box_get_active(GTK_COMBO_BOX(object));
	gboolean	isSens=(type==PLAYER_COMPUTER) ? TRUE : FALSE;
	
	if(object==GTK_OBJECT(gnoBang->playerTypeBox[0]))
	{
		gtk_widget_set_sensitive(GTK_WIDGET(gnoBang->playerStrategyBox[0]), isSens);
	}
	else if(object==GTK_OBJECT(gnoBang->playerTypeBox[1]))
	{
		gtk_widget_set_sensitive(GTK_WIDGET(gnoBang->playerStrategyBox[1]), isSens);
	}
	else
	{
		g_warning("typeChangedCB called for unknown object");
		return;
	}
}


/************************************************************************
 *																		*
 * opens gnobang preferences file									 	*
 *  																	*
 ************************************************************************/

static FILE *fopenPrefs(const char *mode)
{
	FILE	*fh=NULL;
	char	*home=getenv("HOME");
	
	if(home)
	{
		char	*s=g_strdup_printf("%s/%s", home, GNOBANG_PREFS);
		
		fh=fopen(s, mode);
		if(fh==NULL)
		{
			char	*error=strerror(errno);
			g_warning("failed to open gnobang preferences: %s", error);
		}
		
		g_free(s);
	}
	
	return fh;
}


/************************************************************************
 *																		*
 * saves current preferences										 	*
 *  																	*
 ************************************************************************/

static guint32 MARKER=0x476E4F62;

void savePrefs()
{
	FILE	*fh=fopenPrefs("w");
	if(fh)
	{
		fwrite(&MARKER, sizeof(MARKER), 1, fh);
		fwrite(goGame, sizeof(GoGame), 1, fh);
		fclose(fh);
	}
}

/************************************************************************
 *																		*
 * loades current preferences										 	*
 *  																	*
 ************************************************************************/

void loadPrefs()
{
	FILE	*fh;
	
	// init with defaults
	memcpy(goGame, &DefaultGoGame, sizeof(GoGame));
	
	fh=fopenPrefs("r");
	if(fh)
	{
		guint32	marker;
		fread(&marker, sizeof(marker), 1, fh);
		
		if(marker==MARKER)
		{
			GoGame	gg;
			
			if(fread(&gg, sizeof(GoGame), 1, fh)==1)
			{
				memcpy(goGame, &gg, sizeof(GoGame));
			}
			else
			{
				char	*error=strerror(errno);
				g_warning("failed to load gnobang preferences: %s", error);
			}
		}

		fclose(fh);
	}
}

