/* GnoBang: (c) 2008 by Andreas S. Krebs
 * 
 * This file is part of GnoBang.
 *
 * GnoBang is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GnoBang is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GnoBang.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "gnobang.h"

#ifdef SOUND_SUPPORT
#include <gst/gst.h>
#endif


/************************************************************************
 *																		*
 * init the sound system											 	*
 *  																	*
 ************************************************************************/

void soundSystemInit(int *pargc, char **pargv[])
{
#ifdef SOUND_SUPPORT
	gst_init (pargc, pargv);
#endif
}


/************************************************************************
 *																		*
 * start playing of a sound											 	*
 *  																	*
 ************************************************************************/

#ifdef SOUND_SUPPORT
static gboolean startPlaying(gpointer data)
{
	GstElement	*play=GST_ELEMENT(data);
	GstBus		*bus=gst_pipeline_get_bus(GST_PIPELINE(play));
	GstMessage	*msg;
	
	// wait for playback to finish or fail
	msg=gst_bus_poll(bus, GST_MESSAGE_EOS|GST_MESSAGE_ERROR, -1);
	gst_message_unref(msg);	

	// cleanup
	gst_object_unref(bus);	
	gst_element_set_state(play, GST_STATE_NULL);
	gst_object_unref(GST_OBJECT(play));
	
	return FALSE;
}
#endif


/************************************************************************
 *																		*
 * Plays the requested sound.											*
 *  																	*
 ************************************************************************/

void playSound(char *soundname, gboolean block)
{
#ifdef SOUND_SUPPORT
	GstElement	*play;
	gchar		*uri;

	if(!goGame->SoundOn)
		return;
	
	// setup stream
	uri=g_strdup_printf("file://%s/%s", SOUNDS_DIR, soundname);
	play=gst_element_factory_make("playbin", "play");
	g_object_set(G_OBJECT(play), "uri", uri, NULL);
	g_free(uri);

	// start playing
	gst_element_set_state(play, GST_STATE_PLAYING);
	
	if(block)
	{
		startPlaying(play);
	}
	else
	{
		g_idle_add(startPlaying, play);
	}
#endif
}


/************************************************************************
 *																		*
 * cleanup sound system													*
 *  																	*
 ************************************************************************/

void soundSystemCleanup()
{
#ifdef SOUND_SUPPORT
	// nothing to be done
#endif
}
